<?php

namespace Quivers\Components;

use Shopware\Components\HttpClient\GuzzleFactory;
use Shopware\Components\HttpClient\GuzzleHttpClient;
use Quivers\Components\SettingService;

class LogTracking
{

    public $logEndpoint = 'logs/';
    public $client;
    public $settingsService;
        
    public function __construct(GuzzleFactory $factory, SettingService $settingsService)
    {
        $this->client = new GuzzleHttpClient($factory);
        $this->settingsService = $settingsService;
     }

    public function session_start($order_detail, $time){
        $logData = [
            "order_detail" => $order_detail,
            "time" => $time
        ];
         $this->logging("SESSION START", $logData);
    }

    public function validate_api_call($request, $response, $start_time, $end_time){
        $logData = [
            "start_time" => $start_time,
            "end_time" => $end_time,
            "request" => $request,
            "response" => $response
        ];
        $this->logging("VALIDATE API CALL", $logData);
    }

    public function countries_api_call($request, $response, $start_time, $end_time){
         $logData = [
                "request" => $request,
                "response" => $response,
                "start_time" => $start_time,
                "end_time" => $end_time
            ];
         $this->logging("COUNTRIES API CALL", $logData);
    }

    public function payment_request($order_data, $payment_method, $time, $response, $success){
         $logData = [
                "order_data" => $order_data,
                "payment_method" => $payment_method,
                "time" => $time,
                "response" => $response,
                "success" => $success
            ];
         $this->logging("PAYMENT REQUEST", $logData);
    }

    public function session_end($order_detail, $time){
         $logData = [
                "order_detail" => $order_detail,
                "time" => $time
            ];
         $this->logging("SESSION END", $logData);
    }

    public function captcha_data($token ,$captcha_response){
        $logData = [
                "token" => $token,
                "captcha_response" => $captcha_response
            ];
        $this->logging("CAPTCHA RESPONSE", $logData);
    }

    public function print_statement($key){
        $logData = [
                "message" => $key
            ];
        $this->logging("MESSAGE", $logData);
    }

    public function logging($type, $data)
    {
        $quiversSandbox = (bool)$this->settingsService->get('sandbox', 'general');
        $quiversAPIKey = $this->settingsService->get('api_key', 'general');
        
        $baseUrl = 'https://middleware.quiversdemo.com/profile/';
        if(!$quiversSandbox) {
            $baseUrl = 'https://middleware.quivers.com/profile/';
        }
        $logdata = [
            "client_type" => "Shopware",
            "api_key" => "apiKey ".$quiversAPIKey,
            "uuid" => uniqid(),
            "type" => $type,
            "log_data" => $data
        ];
        $logUrl = $baseUrl.$this->logEndpoint;  
        $response = $this->client->post($logUrl, [], json_encode($logdata));
    }


}