<?php

namespace Quivers\Components;

use Shopware\Components\HttpClient\GuzzleHttpClient as GuzzleClient;
use Shopware\Components\HttpClient\GuzzleFactory;
use Quivers\Components\SettingService;

class QuiversConfig
{
    protected $baseUrl = 'https://middleware.quiversdemo.com/profile';
    protected $headers;
    protected $client;
    private $settingsService;


    public function __construct(GuzzleFactory $factory, SettingService $settingsService){
        $this->client = new GuzzleClient($factory);
        $this->settingsService = $settingsService;
        $this->BackendSession = Shopware()->BackendSession();
    } 


    public function profileCreate(array $settings){
        $apiKey = $settings['apikey'];
        $quiversSandbox = $settings['sandbox'];
        $shopwareBaseUrl = $settings['baseUrl'];
        $businessRefId = $settings['businessRefId'];
        $basicAuth = $settings['basicAuth'];
        $enablePlugin = $settings['enablePlugin'];
        
        if (filter_var($shopwareBaseUrl, FILTER_VALIDATE_URL) === false) {
            return 'Invalid Shopware Base URL Format. If the issue still persists, please contact enterprise@quivers.com
            for further assistance.';
        }
        if($quiversSandbox=='false'){
            if(substr($shopwareBaseUrl, 0, 5)!=="https"){
                return 'Invalid URL. If test mode is disabled, please set the base URL to HTTPS and try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.';
            }
        }

        $message = $this->validate_shopware_creds($settings);
        if($message != 'success'){
            return $message;
        }
        if($quiversSandbox == 'false') {
            $this->baseUrl = "https://middleware.quivers.com/profile";
        }

        $profileUrl = $this->baseUrl . "/create";

        $payload = [
            "client_type" => "Shopware",
            "base_url" => $shopwareBaseUrl,
            "business_refid" => $businessRefId,
            "api_key" => $apiKey,
            "basic_auth"=>$basicAuth,
            "enable_plugin"=>$enablePlugin
        ];

        $this->headers['content-type'] ='application/json';
        try{
            $response = $this->client->post($profileUrl, $this->headers, json_encode($payload));
        }
        catch(\Exception $e){
            $error = $e->getBody();
            $error = str_replace(array('{','}',':', '"','error'), '',$error);
            return "Unable to sync to Quivers. Error:".$error;
        }
        $responseBody = $response->getBody();
        $responseDecoded = json_decode($responseBody, true);
        $uuid = $responseDecoded["uuid"];
        $this->BackendSession["profile_uuid"] = $uuid;
        return 'success';
    }

    public function validate_shopware_creds(array $settings){
        $shopwareBaseUrl = $settings['baseUrl'];
        $basicAuth = $settings['basicAuth'];
        $endpoint = 'api/orders';
        $url = $shopwareBaseUrl.$endpoint;
        $shopwareHeaders["Authorization"] = $basicAuth;
        try{
            $response = $this->client->get($url, $shopwareHeaders);
                return 'success';
        }
        catch(\Exception $e){
            $code = $e->getCode();
            if($code == 404){
                return 'Invalid Shopware Base Url. Please try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.';
            }
            if($code == 401){
                return 'Invalid Basic Authorization. Please try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.';
            }
            return 'Invalid Shopware Base Url or Basic Authorization token. Please try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.';
        }

    }

    public function profile_update($params){
        $apiKey = $this->settingsService->get('api_key', 'general');
        $quiversSandbox = $this->settingsService->get('sandbox', 'general');
        if(!$quiversSandbox) {
            $this->baseUrl = "https://middleware.quivers.com/profile";
        }
        $profileUpdateUrl = $this->baseUrl . "/update/marketplaces";


        $usdMarketPlaceId = $params['USD_Marketplace'];
        $euroMarketPlaceId = $params['EUR_Marketplace'];

        $cadMarketPlaceId = $params['CAD_Marketplace'];
        $audMarketPlaceId = $params['AUD_Marketplace'];
        $jpyMarketPlaceId = $params['JPY_Marketplace'];
        $gbpMarketPlaceId = $params['GBP_Marketplace'];

        $currencies = ["USD", "EUR", "AUD", "JPY", "GBP", "CAD"];
        $marketplaces_request_data = [];
        foreach ($currencies as $currency) {
            if($currency == 'USD'){
                $marketplace = $params['USD_Marketplace'];
                $claimingGroups = $params["USD_Claiming"];
            }
            if($currency == 'EUR'){
                $marketplace = $params['EUR_Marketplace'];
                $claimingGroups = $params["EUR_Claiming"];
            }
            if($currency == 'AUD'){
                $marketplace =$params['AUD_Marketplace'];
                $claimingGroups = $params["AUD_Claiming"];
            }
            if($currency == 'JPY'){
                $marketplace =$params['JPY_Marketplace'];
                $claimingGroups = $params["JPY_Claiming"];
            }
            if($currency == 'GBP'){
                $marketplace =$params['GBP_Marketplace'];
                $claimingGroups = $params["GBP_Claiming"];
            }
            if($currency == 'CAD'){
                $marketplace =$params['CAD_Marketplace'];
                $claimingGroups = $params["CAD_Claiming"];
            }
            if(!$marketplace) continue;

            if (sizeof($claimingGroups)>1){
                $claimingGroups = $this->removeEmptyClaimingGroup($claimingGroups);
            }
            if (!$claimingGroups){
                $claimingGroups = [];
            }

            $data = [];
            $data['currency'] = $currency;
            $data['marketplace_id'] = $marketplace;
            $data['claiming_group_ids'] = $claimingGroups;
            array_push($marketplaces_request_data, $data);
        }


        $request_data = [
            "marketplaces" => $marketplaces_request_data,
        ];

        $this->headers['content-type'] ='application/json';
        $this->headers['uuid'] = $this->BackendSession["profile_uuid"];
        try{
            $response = $this->client->post($profileUpdateUrl, $this->headers, json_encode($request_data));
        }
        catch(\Exception $e){
            $error = $e->getBody();
            $error = str_replace(array('{','}',':', '"','error'), '',$error);
            return "Unable to sync Quivers Profile. Please verify that your are connected to Quivers. Error : ".$error. 'If the issue still persists, please contact enterprise@quivers.com for further assistance.';
        }
        $responseBody = $response->getBody();
        
        return 'success';
    }


    public function removeEmptyClaimingGroup($claimingGroup){
        $result = $claimingGroup;
        if (($key = array_search("", $claimingGroup)) !== false) {
            $firstHalf = array_slice($claimingGroup,0, $key);
            $secondHalf = array_slice($claimingGroup, $key);
            unset($secondHalf[0]);
            $result = array_merge($firstHalf, $secondHalf);
        }
        return $result;

    }

}
