<?php

use Quivers\Models\Settings\QuiversTaxSettings;

class Shopware_Controllers_Backend_QuiversTaxSettings extends Shopware_Controllers_Backend_Application
{
    protected $model = QuiversTaxSettings::class;

    public function detailAction()
    {
        $settings = $this->getSettings();
        if ($settings !== null) {
            $this->view->assign('tax_settings', $settings);
        }
        else{
            $settings = [];
            $this->view->assign('tax_settings', $settings);
        }
    }

    public function getSettings()
    {
        /** @var QuiversTaxSettings $taxSettings */
        $allRecords = [];
        $taxSettings = $this->getRepository()->findAll();
        if(count($taxSettings)){
            $allRecords['id'] = 1;
        }
        else{
            $allRecords['id'] =0;
        }

        foreach($taxSettings as $row) {
            if($row->currency=='USD') {
                $allRecords['USD_Marketplace'] = $row->marketPlaces;
                $allRecords['USD_Claiming'] = $row->claimingGroups;
            }
            if($row->currency=='EUR') {
                $allRecords['EUR_Marketplace'] = $row->marketPlaces;
                $allRecords['EUR_Claiming'] = $row->claimingGroups;
            }
            if($row->currency=='AUD') {
                $allRecords['AUD_Marketplace'] = $row->marketPlaces;
                $allRecords['AUD_Claiming'] = $row->claimingGroups;
            }
            if($row->currency=='JPY') {
                $allRecords['JPY_Marketplace'] = $row->marketPlaces;
                $allRecords['JPY_Claiming'] = $row->claimingGroups;
            }
            if($row->currency=='GBP') {
                $allRecords['GBP_Marketplace'] = $row->marketPlaces;
                $allRecords['GBP_Claiming'] = $row->claimingGroups;
            }
            if($row->currency=='CAD') {
                $allRecords['CAD_Marketplace'] = $row->marketPlaces;
                $allRecords['CAD_Claiming'] = $row->claimingGroups;
            }
            
        }

        $service = $this->container->get('quivers_plugin.marketplaces_config');
        if (!$service){
            return $allRecords;        
        }

        //Call Claiming policy API
        $taxSettings = $service->getClaimingPolicies();
        $claimingPolicy = $taxSettings['claimingPolicy'];
        $validClaimingPolicy = $this->validateClaimingPolicy($claimingPolicy);

        //Remove deleted and blacklisted claiming policy
        foreach ($allRecords as $key => $value) {
            $validIds = [];
            if (strpos($key, 'Claiming')){
                foreach ($value as $Claiming) {
                    if (!in_array($Claiming, $validClaimingPolicy, true)) {
                        continue;
                    }
                    array_push($validIds, $Claiming);
                }
                if (empty($validIds)){
                    array_push($validIds, "");
                }
                $allRecords[$key] = $validIds;
            }   
        }

        return $allRecords;
    }

    public function validateClaimingPolicy($claimingPolicy){
        $ids = [];
        array_push($ids, "");
        foreach ($claimingPolicy as $value) {
            array_push($ids, (string)$value->id);
        }
        return $ids;
    }

    public function createAction(){
        $allRecords = [];
        //USD market places and currencies
        $USD_Marketplace = $this->Request()->getParam("USD_Marketplace");
        $USD_Claiming = $this->Request()->getParam("USD_Claiming");
        $USD_Claiming = sizeof($USD_Claiming)>1? $this->removeEmptyClaimingGroup($USD_Claiming):  $USD_Claiming;
        if (sizeof($USD_Claiming)==0){
            $USD_Claiming = [""];
        } 
        $record = [
            'currency' => 'USD',
            'marketPlaces' => $USD_Marketplace,
            'claimingGroups' => $USD_Claiming 
        ];
        $allRecords[] = $record;
        
        //EURO market places and currencies
        $EUR_Marketplace = $this->Request()->getParam("EUR_Marketplace");
        $EUR_Claiming = $this->Request()->getParam("EUR_Claiming");
        $EUR_Claiming = sizeof($EUR_Claiming)>1? $this->removeEmptyClaimingGroup($EUR_Claiming):  $EUR_Claiming;
        if (sizeof($EUR_Claiming)==0){
            $EUR_Claiming = [""];
        }
        $record = [
            'currency' => 'EUR',
            'marketPlaces' => $EUR_Marketplace,
            'claimingGroups' => $EUR_Claiming 
        ];
        $allRecords[] = $record;

        //CAD market places and currencies
        $CAD_Marketplace = $this->Request()->getParam("CAD_Marketplace");
        $CAD_Claiming = $this->Request()->getParam("CAD_Claiming");
        $CAD_Claiming = sizeof($CAD_Claiming)>1? $this->removeEmptyClaimingGroup($CAD_Claiming):  $CAD_Claiming;
        if (sizeof($CAD_Claiming)==0){
            $CAD_Claiming = [""];
        }
        $record = [
            'currency' => 'CAD',
            'marketPlaces' => $CAD_Marketplace,
            'claimingGroups' => $CAD_Claiming 
        ];
        $allRecords[] = $record;


        //JPY market places and currencies
        $JPY_Marketplace = $this->Request()->getParam("JPY_Marketplace");
        $JPY_Claiming = $this->Request()->getParam("JPY_Claiming");
        $JPY_Claiming = sizeof($JPY_Claiming)>1? $this->removeEmptyClaimingGroup($JPY_Claiming):  $JPY_Claiming;
        if (sizeof($JPY_Claiming)==0){
            $JPY_Claiming = [""];
        }
        $record = [
            'currency' => 'JPY',
            'marketPlaces' => $JPY_Marketplace,
            'claimingGroups' => $JPY_Claiming 
        ];
        $allRecords[] = $record;
        
                //AUD market places and currencies
        $AUD_Marketplace = $this->Request()->getParam("AUD_Marketplace");
        $AUD_Claiming = $this->Request()->getParam("AUD_Claiming");
        $AUD_Claiming = sizeof($AUD_Claiming)>1? $this->removeEmptyClaimingGroup($AUD_Claiming):  $AUD_Claiming;
        if (sizeof($AUD_Claiming)==0){
            $AUD_Claiming = [""];
        }
        $record = [
            'currency' => 'AUD',
            'marketPlaces' => $AUD_Marketplace,
            'claimingGroups' => $AUD_Claiming 
        ];
        $allRecords[] = $record;
        
          //GBP market places and currencies
        $GBP_Marketplace = $this->Request()->getParam("GBP_Marketplace");
        $GBP_Claiming = $this->Request()->getParam("GBP_Claiming");
        $GBP_Claiming = sizeof($GBP_Claiming)>1? $this->removeEmptyClaimingGroup($GBP_Claiming):  $GBP_Claiming;
        if (sizeof($GBP_Claiming)==0){
            $GBP_Claiming = [""];
        }
        $record = [
            'currency' => 'GBP',
            'marketPlaces' => $GBP_Marketplace,
            'claimingGroups' => $GBP_Claiming 
        ];
        $allRecords[] = $record;
        foreach($allRecords as $record) {
            $this->save($record);
        }
    }

    public function removeEmptyClaimingGroup($claimingGroup){
        $result = $claimingGroup;
        if (($key = array_search("", $claimingGroup)) !== false) {
            $firstHalf = array_slice($claimingGroup,0, $key);
            $secondHalf = array_slice($claimingGroup, $key);
            unset($secondHalf[0]);
            $result = array_merge($firstHalf, $secondHalf);
        }
        return $result;

    }

    public function updateAction(){
        $taxRecord = $this->getRepository()->findAll();
        if($taxRecord === null){
            return;
        }
        $modifiedRecords = [];
        foreach ($taxRecord as $record){
            $USD_Marketplace = $this->Request()->getParam("USD_Marketplace");
            $USD_Claiming = $this->Request()->getParam("USD_Claiming");
            $USD_Claiming = sizeof($USD_Claiming)>1? $this->removeEmptyClaimingGroup($USD_Claiming):  $USD_Claiming;
            if (sizeof($USD_Claiming)==0){
                $USD_Claiming = [""];
            }
            if($record->currency == 'USD')  { 
                $record->marketPlaces = $USD_Marketplace;
                $record->claimingGroups = $USD_Claiming;
                $modifiedRecords[] = (array)$record;
               
            }
            //EURO market places and currencies
            $EUR_Marketplace = $this->Request()->getParam("EUR_Marketplace");
            $EUR_Claiming = $this->Request()->getParam("EUR_Claiming");
            $EUR_Claiming = sizeof($EUR_Claiming)>1? $this->removeEmptyClaimingGroup($EUR_Claiming):  $EUR_Claiming;
            if (sizeof($EUR_Claiming)==0){
                $EUR_Claiming = [""];
            }
            if($record->currency == 'EUR')  {              
                $record->marketPlaces = $EUR_Marketplace;
                $record->claimingGroups = $EUR_Claiming;
                $modifiedRecords[] = (array)$record;
            }

            //CAD market places and currencies
            $CAD_Marketplace = $this->Request()->getParam("CAD_Marketplace");
            $CAD_Claiming = $this->Request()->getParam("CAD_Claiming");
            $CAD_Claiming = sizeof($CAD_Claiming)>1? $this->removeEmptyClaimingGroup($CAD_Claiming):  $CAD_Claiming;
            if (sizeof($CAD_Claiming)==0){
                $CAD_Claiming = [""];
            }
            if($record->currency == 'CAD')    {            
                $record->marketPlaces = $CAD_Marketplace;
                $record->claimingGroups = $CAD_Claiming;
                $modifiedRecords[] = (array)$record;
            }


            //JPY market places and currencies
            $JPY_Marketplace = $this->Request()->getParam("JPY_Marketplace");
            $JPY_Claiming = $this->Request()->getParam("JPY_Claiming");
            $JPY_Claiming = sizeof($JPY_Claiming)>1? $this->removeEmptyClaimingGroup($JPY_Claiming):  $JPY_Claiming;
            if (sizeof($JPY_Claiming)==0){
                $JPY_Claiming = [""];
            }
            if($record->currency == 'JPY')  {              
                $record->marketPlaces = $JPY_Marketplace;
                $record->claimingGroups = $JPY_Claiming;
                $modifiedRecords[] = (array)$record;
            }

            //AUD market places and currencies
            $AUD_Marketplace = $this->Request()->getParam("AUD_Marketplace");
            $AUD_Claiming = $this->Request()->getParam("AUD_Claiming");
            $AUD_Claiming = sizeof($AUD_Claiming)>1? $this->removeEmptyClaimingGroup($AUD_Claiming):  $AUD_Claiming;
            if (sizeof($AUD_Claiming)==0){
                $AUD_Claiming = [""];
            }
            if($record->currency == 'AUD')  {              
                $record->marketPlaces = $AUD_Marketplace;
                $record->claimingGroups = $AUD_Claiming;
                $modifiedRecords[] = (array)$record;
            }
            //GBP market places and currencies
            $GBP_Marketplace = $this->Request()->getParam("GBP_Marketplace");
            $GBP_Claiming = $this->Request()->getParam("GBP_Claiming");
            $GBP_Claiming = sizeof($GBP_Claiming)>1? $this->removeEmptyClaimingGroup($GBP_Claiming):  $GBP_Claiming;
            if (sizeof($GBP_Claiming)==0){
                $GBP_Claiming = [""];
            }
            if($record->currency == 'GBP')  {              
                $record->marketPlaces = $GBP_Marketplace;
                $record->claimingGroups = $GBP_Claiming;
                $modifiedRecords[] = (array)$record;
            }
        }
        foreach($modifiedRecords as $record) {
            $this->save($record);
        }

    }

    public function profileUpdateAction(){
        $service = $this->container->get('quivers_plugin.quivers_config');
        if (!$service){
            $this->View()->assign([
                'success' => false,
                'message' => "Unable to sync Quivers Profile. Please verify that your are connected to Quivers. If the issue still persists, please contact enterprise@quivers.com for further assistance.",
            ]);
            return;
        }
        $profileParams = $this->validateProfileParams($this->Request()->getParams());
        $message = $service->profile_update($profileParams);
        if($message != 'success'){
            $this->View()->assign([
                'success' => false,
                'message' => $message,
            ]);
            return;
        }
        $this->View()->assign('success', true);
    }

    public function validateProfileParams($params) {
        $validatedParams = [];
        foreach($params as $key=>$value) {
            if(!$value) continue;
            if(strpos($key, 'Marketplace')) $validatedParams[$key] = $value;
            elseif(strpos($key, 'Claiming')) $validatedParams[$key] = explode(',', $value);
        }
        return $validatedParams;
    }

}