// {namespace name="backend/quivers_configuration/main"}
// {block name="backend/quivers_configuration/controller/main"}
Ext.define('Shopware.apps.QuiversConfiguration.controller.Main', {
    extend: 'Enlight.app.Controller',

    /**
     * @type { Shopware.apps.QuiversConfiguration.view.Window }
     */
    window: null,

    /**
     * @type { Boolean }
     */
    settingsSaved: false,

    /**
     * @type { String }
     */
    validateUrl: '{url controller=QuiversConfiguration action=validate}',
    /**
     * @type { String }
     */
    upcUrl: '{url controller=QuiversConfiguration action=upc}',
    /**
     * @type { String }
     */
    stripeDetailUrl: '{url controller=StripeSettings action=detail}',
    
    /**
     * @type { String }
     */
    generalDetailUrl: '{url controller=QuiversConfiguration action=detail}',

    /**
     * @type { String }
     */
    taxDetailUrl: '{url controller=QuiversTaxSettings action=detail}',

    /**
     * @type { String }
     */
    profileUpdateUrl: '{url controller=QuiversTaxSettings action=profileUpdate}',

     /**    
     * @type { Shopware.apps.QuiversConfiguration.model.StripeSettings }
     */
    stripeRecord: null,

     /**    
     * @type { Shopware.apps.QuiversConfiguration.model.QuiversSetting }
     */
    generalRecord: null,

    /**    
     * @type { Shopware.apps.QuiversConfiguration.model.QuiversTaxSettings }
     */
    taxRecord: null,

    records: null,

    refs: [
        { ref: 'GeneralTab', selector: 'quivers-configuration-tabs-general' },
        { ref: 'StripeTab', selector: 'quivers-configuration-tabs-stripe-config' },
        { ref: 'TaxTab', selector: 'quivers-configuration-tabs-quivers-tax'}
    ],

    init: function() {
        var me = this;

        me.createMainWindow();
        me.createComponentControl();
        me.beforeLoadDetails();

        me.callParent(arguments);
    },

    beforeLoadDetails: function() {
        var me = this;
        me.checkForStores(function(result) {
            if(result) me.loadDetails();
            me.window.unmask();
            me.window.setLoading(false);
        });
    },

    checkForStores: function(callback) {
        var me = this;
        var myComboStores = ['QuiversConfigurationMarketplaces', 'QuiversConfigurationClaimingGroup'];
        var loaded = true;
        Ext.each(myComboStores, function(storeId) {
            var store = Ext.getStore(storeId);
            if (store.isLoading()) {
                loaded = false;
            }
        });
        if(loaded) {
            return callback(loaded);
        }
        else {
            setTimeout(function() {
                me.checkForStores(callback);
            }, 500);
        }
    },

    createComponentControl: function() {
        var me = this;

        me.control({
            'quivers-configuration-tabs-stripe-config': {
                saveStripeSettings: me.onSaveStripeSettings
            },
            'quivers-configuration-tabs-general':{
                saveQuiversSettings: me.onSaveQuiversSettings
            },
            'quivers-configuration-tabs-quivers-tax':{
                saveTaxSettings: me.onSaveTaxSettings
            }

        });
    },


    createMainWindow: function() {
        var me = this;
        me.window = me.getView('Window').create().show();
        me.window.mask('loading');
        me.window.setLoading('Loading settings...');
    },

    loadDetails: function() {
        var me = this;
        
        me.prepareRecords();
        me.loadSetting(me.stripeDetailUrl);
        me.loadSetting(me.generalDetailUrl);
        me.loadSetting(me.taxDetailUrl);

    },

    loadSetting: function(detailUrl) {
        var me = this;
        
        Ext.Ajax.request({
            url: detailUrl,
            callback: Ext.bind(me.onDetailAjaxCallback, me)
        });
    },

    prepareRecords: function() {
        var me = this,
            stripeTab = me.getStripeTab(),
            generalTab = me.getGeneralTab(),
            taxTab = me.getTaxTab();

        me.stripeRecord = Ext.create('Shopware.apps.QuiversConfiguration.model.StripeSettings');
        me.generalRecord = Ext.create('Shopware.apps.QuiversConfiguration.model.QuiversSetting');
        me.taxRecord = Ext.create('Shopware.apps.QuiversConfiguration.model.QuiversTaxSettings');

        stripeTab.loadRecord(me.stripeRecord);
        generalTab.loadRecord(me.generalRecord);
        taxTab.loadRecord(me.taxRecord);

    },

    onSaveStripeSettings: function() {
        var me = this,
            stripeTabForm = me.getStripeTab().getForm(),
            stripeSettings = stripeTabForm.getValues();

        if (!stripeTabForm.isValid()) {
            Shopware.Notification.createGrowlMessage('Quivers', 'Please fill out all fields marked in red.', me.window.title);
            return;
        }

        me.window.setLoading('Saving settings...');
        me.stripeRecord.set(stripeSettings);

        me.stripeRecord.save();

        Shopware.Notification.createGrowlMessage('Quivers', 'The settings have been saved!', me.window.title);
        me.window.setLoading(false);

    },

    
    onSaveTaxSettings: function(){
        var me = this,
            taxTabForm = me.getTaxTab().getForm(),
            taxSettings = taxTabForm.getValues();

        if (!taxTabForm.isValid()) {
            Shopware.Notification.createGrowlMessage('Quivers', 'Please fill out all fields marked in red.', me.window.title);
            return;
        }

        me.window.setLoading('Saving settings...');
        me.taxRecord.set(taxSettings);

        me.taxRecord.save();

        Ext.Ajax.request({
            url: me.profileUpdateUrl,
            params: {
                USD_Marketplace: taxSettings.USD_Marketplace,
                CAD_Marketplace: taxSettings.CAD_Marketplace,
                AUD_Marketplace: taxSettings.AUD_Marketplace,
                JPY_Marketplace: taxSettings.JPY_Marketplace,
                EUR_Marketplace: taxSettings.EUR_Marketplace,
                GBP_Marketplace: taxSettings.GBP_Marketplace,
                USD_Claiming: taxSettings.USD_Claiming.join(','),
                CAD_Claiming: taxSettings.CAD_Claiming.join(','),
                AUD_Claiming: taxSettings.AUD_Claiming.join(','),
                JPY_Claiming: taxSettings.JPY_Claiming.join(','),
                EUR_Claiming: taxSettings.EUR_Claiming.join(','),
                GBP_Claiming: taxSettings.GBP_Claiming.join(',')
            },
            callback: Ext.bind(me.onProfileUpdateAjaxCallback, me)
        });
    },

    onSaveQuiversSettings: function(){
        var me = this,
            generalTabForm = me.getGeneralTab().getForm(),
            generalSettings = generalTabForm.getValues();

        if (!generalTabForm.isValid()) {
            Shopware.Notification.createGrowlMessage('Quivers', 'Please fill out all fields marked in red.', me.window.title);
            return;
        }

        me.window.setLoading('Saving settings...');
        Ext.Ajax.request({
            url: me.upcUrl,
            params: {
                upcField: generalSettings["upcField"]
            },
            callback: Ext.bind(me.onUpcAjaxCallback, me)
        });


    },

    /**
     * @param { Object } options
     * @param { Boolean } success
     * @param { Object } response
     */
    onDetailAjaxCallback: function(options, success, response) {
        var me = this;

        if (!success) {
            Shopware.Notification.createGrowlMessage('Quivers', 'Could not load settings due to an unknown error', me.window.title);
        }

        var stripeTab = me.getStripeTab(),
            generalTab = me.getGeneralTab(),
            taxTab = me.getTaxTab(),
            settings = Ext.JSON.decode(response.responseText);
       if (settings.stripe_settings) {
            me.stripeRecord = Ext.create('Shopware.apps.QuiversConfiguration.model.StripeSettings', settings.stripe_settings);
            stripeTab.loadRecord(me.stripeRecord);
        }
        else if (settings.general_settings) {
            me.generalRecord = Ext.create('Shopware.apps.QuiversConfiguration.model.QuiversSetting', settings.general_settings);
            generalTab.loadRecord(me.generalRecord);
        }
        else if (settings.tax_settings) {
            var tax = settings.tax_settings;
            me.taxRecord = Ext.create('Shopware.apps.QuiversConfiguration.model.QuiversTaxSettings', settings.tax_settings);
            taxTab.loadRecord(me.taxRecord);
        }  
  
        me.settingsSaved = false;
    },

     /**
     * @param { Object } options
     * @param { Boolean } success
     * @param { Object } response
     */
    onValidateAjaxCallback: function(options, success, response) {
        var me = this,
            responseObject = Ext.JSON.decode(response.responseText),
            message = '';

         if (Ext.isDefined(responseObject) && responseObject.success) {
            Shopware.Notification.createGrowlMessage('Quivers', 'The settings are valid.', me.window.title);
            me.window.setLoading(false);

            return;
        }
        if (Ext.isDefined(responseObject)) {
            message = responseObject.message;
        }
        Shopware.Notification.createStickyGrowlMessage(
            {
                title: 'Quivers',
                text: 'The settings are invalid' + '<br><u>' + message + '</u>'
            },
            me.window.title
        );

        me.window.setLoading(false);
    },

    /**
     * @param { Object } options
     * @param { Boolean } success
     * @param { Object } response
     */
    onUpcAjaxCallback: function(options, success, response) {
        var me = this,
            responseObject = Ext.JSON.decode(response.responseText),
            message = '';

         if (Ext.isDefined(responseObject) && responseObject.success) {
                var generalTabForm = me.getGeneralTab().getForm(),
                generalSettings = generalTabForm.getValues();
                me.generalRecord.set(generalSettings);
                me.generalRecord.save();
                me.window.setLoading(false);
                me.window.setLoading('Validating settings...');
                Ext.Ajax.request({
                    url: me.validateUrl,
                    params: {
                        apiKey: generalSettings["apiKey"],
                        businessRefId: generalSettings["businessRefId"],
                        BaseUrl: generalSettings["baseUrl"],
                        basicAuth: generalSettings["basicAuth"],
                        upcField: generalSettings["upcField"],
                        sanbox: generalSettings["sandbox"],
                        enablePlugin: generalSettings["enablePlugin"]
                    },
                    callback: Ext.bind(me.onValidateAjaxCallback, me)
                });
        }
    },

     /**
     * @param { Object } options
     * @param { Boolean } success
     * @param { Object } response
     */
    onProfileUpdateAjaxCallback: function(options, success, response) {
        var me = this,
            responseObject = Ext.JSON.decode(response.responseText),
            message = '';

         if (Ext.isDefined(responseObject) && responseObject.success) {
            Shopware.Notification.createGrowlMessage('Quivers', 'The settings are valid.', me.window.title);
            me.window.setLoading(false);

            return;
        }
        if (Ext.isDefined(responseObject)) {
            message = responseObject.message;
        }
        Shopware.Notification.createStickyGrowlMessage(
            {
                title: 'Quivers',
                text: 'The settings are invalid' + '<br><u>' + message + '</u>'
            },
            me.window.title
        );

        me.window.setLoading(false);
    }

    

});
// {/block}
