Ext.define('Shopware.apps.QuiversConfiguration.view.tabs.StripeConfig', {
    extend: 'Ext.form.Panel',
    alias: 'widget.quivers-configuration-tabs-stripe-config',
    title: 'Stripe Settings',

    anchor: '100%',
    border: false,
    bodyPadding: 10,

    style: {
        background: '#EBEDEF'
    },

    fieldDefaults: {
        anchor: '100%',
        labelWidth: 180
    },

    /**
     * @type { Ext.form.FieldSet }
     */
    restContainer: null,


    initComponent: function () {
        var me = this;

        me.items = me.createItems();

        me.callParent(arguments);

        // Manually set the background color of the toolbar.
        me.toolbarContainer.setBodyStyle(me.style);
    },

    registerEvents: function () {
        var me = this;

        me.addEvents(
            /**
             * Will be fired when the user clicks on the save button
             */
            'saveStripeSettings'
        );
    },
    /**
     * @returns { Array }
     */
    createItems: function () {
        var me = this;

        return [
            me.createSettings()
        ];
    },

    createSettings: function () {
        var me = this;
        me.toolbarContainer = me.createToolbar();

        me.restContainer = Ext.create('Ext.form.FieldSet', {

            items: [          
                {
                    xtype: 'textfield',
                    name: 'publicKey',
                    fieldLabel: 'Public Key',
                    allowBlank: false
                },
                {
                    xtype: 'textfield',
                    name: 'secretKey',
                    fieldLabel: 'Secret Key',
                    allowBlank: false
                },
                {
                    xtype: 'numberfield',
                    name: 'minOrderTotal',
                    fieldLabel: 'Minimum Order Total',
                    decimalPrecision: 3,
                    supportText: '$0.50 is the minimum amount allowed by Stripe',
                    allowBlank: false
                },
                me.toolbarContainer
                
            ]
        });

        return me.restContainer;   
    },

    createToolbar: function () {
        var me = this;

        return Ext.create('Ext.form.Panel', {
            dock: 'bottom',
            border: false,
            bodyPadding: 5,
            name: 'toolbarContainer',

            items: [{
                xtype: 'button',
                cls: 'primary',
                text: 'Save',
                style: {
                    float: 'right'
                },
                handler: Ext.bind(me.onSaveButtonClick, me)
            }]
        });
    },

    onSaveButtonClick: function () {
        var me = this;

        me.fireEvent('saveStripeSettings');
    }
  
});
