<?php

namespace Quivers\Subscriber;

use Enlight\Event\SubscriberInterface;
use Enlight_Controller_ActionEventArgs as ActionEventArgs;
use Quivers\Components\Position;
use Shopware\Bundle\AttributeBundle\Service\DataPersister;
use Shopware\Components\Model\ModelManager;
use Quivers\Components\LogTracking;


class Backend implements SubscriberInterface
{

    public $logTracking;
    public function __construct(Position $position, DataPersister $dataPersistor, ModelManager $modelManager, LogTracking $logTracking)
    {
        $this->position = $position;
        $this->dataPersistor = $dataPersistor;
        $this->modelManager = $modelManager;
        $this->dbalConnection = $modelManager->getConnection();
        $this->logTracking = $logTracking;

    }

     public static function getSubscribedEvents()
    {
        return [
          'Enlight_Controller_Action_PostDispatchSecure_Backend_AttributeData' => 'attributeUpdate',
          'Enlight_Controller_Action_PostDispatchSecure_Backend_Order' => 'positionUpdate',
          'Enlight_Controller_Action_PostDispatchSecure_Api_Articles' => 'updateAttrArticle',
          'Enlight_Controller_Action_PostDispatchSecure_Backend_Article' =>
          'updateAttrBackend',
          'Enlight_Controller_Action_PostDispatchSecure_Api_Variants' => 'updateAttrVariant',
        ];
    }

    public static function attributeUpdate(ActionEventArgs $args){
        $controller = $args->getSubject();
        $request = $controller->Request();
        $allowedActions = ['saveData'];
        $action = $controller->Request()->getActionName();
        $view = $controller->View();
        if (!in_array($action, $allowedActions, true)) {
            return;
        }
        
        if ($request->getParam('_table') == 's_articles_attributes'){
            $foreignKey =$request->getParam("_foreignKey");

            $articleId = $this->dbalConnection->createQueryBuilder()
                    ->select('articleID')
                    ->from('s_articles_details')
                    ->where('id = :id')
                    ->setParameter('id', $foreignKey)
                    ->execute()->fetchColumn();

            $allDetailId = $this->dbalConnection->createQueryBuilder()
                    ->select('id')
                    ->from('s_articles_details')
                    ->where('articleID = :articleID')
                    ->setParameter('articleID', $articleId)
                    ->execute()->fetchAll();

            $taxCode =$request->getParam("__attribute_product_tax_code");
            $quiversStock =$request->getParam("__attribute_quivers_stock");

            $upc_field =  $this->dbalConnection->createQueryBuilder()
                    ->select('upc_field')
                    ->from('quivers_settings')
                    ->execute()->fetchColumn();
            $current_utc_time = gmdate("Y-m-d H:i:s");
            $data = ["product_tax_code" => $taxCode,
                     "quivers_hidden_upc" => $upc_field,
                     "last_modified_at" => $current_utc_time];

            if($quiversStock!="")
            {
                $data["quivers_stock"] = $quiversStock;
            }

            foreach ($allDetailId as $value) {
                 $this->dataPersistor->persist(
                    $data,
                    $request->getParam('_table'),
                    $value['id']
                );
            }
            return;
        }
        $this->position->changedDateUpdate($args);
        $this->position->updateItem($args);
    }

     public static function positionUpdate(ActionEventArgs $args){
        $controller = $args->getSubject();
        $request = $controller->Request();
        $allowedActions = ['savePosition', 'save'];
        $action = $controller->Request()->getActionName();
        $view = $controller->View();
        if (!in_array($action, $allowedActions, true)) {
            return;
        }
        if (($action == 'savePosition') or ($action == 'save')){
              $this->position->changedDateUpdate($args);
        }
        if ($action == 'save'){
            $this->position->positionStatusUpdate($args);
        }
    }

    public static function updateAttrArticle(ActionEventArgs $args)
    {   
        try 
        {   
            
            $controller = $args->getSubject();
            $allowedActions = ['put','post','batch'];
            $action = $controller->Request()->getActionName();
            if (!in_array($action, $allowedActions, true)) {
                return;
            }
            if ($action == 'put')
            {   
                $request = $controller->Request();
                $articleId = $request->getParam('id');
            }
            if ($action == 'post')
            {
                $response=$args->getResponse('id');
                $articleId=json_decode($response,true)["data"]["id"];
            } 
            if ($action == 'batch')
            {
                $response=json_decode($args->getResponse()->getBody(),true)["data"];
                foreach ($response as $data) 
                {
                    try
                    {
                        $articleId=$data["data"]["id"];
                        $this->GenericUpdate($articleId);
                    }
                    catch(\Exception $e)
                    {
                        $message=json_encode(["Product Update From API (Batch) Failed" => $e->getMessage()]);
                        $this->logTracking->print_statement($message);
                        continue;
                    }
                }
                return ;
            }
            $this->GenericUpdate($articleId);
        }
        catch(\Exception $e)
        {
            $message=json_encode(["Product ADD/Update from API Failed" => $e->getMessage()]);
            $this->logTracking->print_statement($message);
            return ;
        }
    }

    public static function updateAttrBackend(ActionEventArgs $args)
    {   
        try
        {
            $controller = $args->getSubject();
            $request = $controller->Request();
            $allowedActions = ['saveConfiguratorSet','deleteDetail'];
            $action = $controller->Request()->getActionName();
            if (!in_array($action, $allowedActions, true)) {
                return;
            }
            $articleId = $request->getParam("articleId");
            $this->GenericUpdate($articleId);
        }
        catch(\Exception $e)
        {   
            $message=json_encode(["Product Variant ADD/Delete from Backend Failed" => $e->getMessage()]);
            $this->logTracking->print_statement($message);
            return ;
        }

    }


    public static function updateAttrVariant(ActionEventArgs $args)
    {   
        try 
        {   
            $controller = $args->getSubject();
            $allowedActions = ['put','post','batch',"batchDelete"];
            $action = $controller->Request()->getActionName();
            if (!in_array($action, $allowedActions, true)) {
                return;
            }
            if ($action == 'put')
            {   
                $request = $controller->Request();
                if ($request->getParam('useNumberAsId')==true)
                {
                 $number = $request->getParam('id');
                 $articleId = $this->dbalConnection->createQueryBuilder()
                    ->select('articleId')
                    ->from('s_articles_details')
                    ->where('ordernumber = :ordernumber')
                    ->setParameter('ordernumber', $number)
                    ->execute()->fetchColumn();
                }
                else
                {
                $id = $request->getParam('id');
                $articleId = $this->dbalConnection->createQueryBuilder()
                    ->select('articleId')
                    ->from('s_articles_details')
                    ->where('id = :id')
                    ->setParameter('id', $id)
                    ->execute()->fetchColumn();
                }
            }
            if ($action == 'post')
            {
                $id=json_decode($args->getResponse()->getBody(),true)["data"]["id"];
                $articleId = $this->dbalConnection->createQueryBuilder()
                    ->select('articleId')
                    ->from('s_articles_details')
                    ->where('id = :id')
                    ->setParameter('id', $id)
                    ->execute()->fetchColumn();
            } 
            if ($action == 'batch' || $action == 'batchDelete')
            {
                $response=json_decode($args->getResponse()->getBody(),true)["data"];
                foreach ($response as $data) 
                {   
                    try
                    {
                        $articleId=$data["data"]["articleId"];
                        $this->GenericUpdate($articleId);
                    }
                    catch(\Exception $e)
                    {
                        $message=json_encode(["Variant Update/Delete(Batch) from API Failed" => $e->getMessage()]);
                        $this->logTracking->print_statement($message);
                        continue;
                    }
                }
                return ;
            }
            $this->GenericUpdate($articleId);
        }
        catch(\Exception $e)
        {  
            $message=json_encode(["Variant CRUD from API Failed" => $e->getMessage()]);
            $this->logTracking->print_statement($message);
            return ;
        }
    }
    
    public function GenericUpdate($articleId)
    {
        try
        {
            $allDetailId = $this->dbalConnection->createQueryBuilder()
                    ->select('id')
                    ->from('s_articles_details')
                    ->where('articleID = :articleID')
                    ->setParameter('articleID', $articleId)
                    ->execute()->fetchAll();
            $id= $allDetailId[0]["id"];
            $table='s_articles_attributes';
            $current_utc_time = gmdate("Y-m-d H:i:s");
            $data = ["last_modified_at" => $current_utc_time];
            $this->dataPersistor->persist(
                $data,
                $table,
                $id
            );
        }
        catch(\Exception $e)
        {   
            $message=json_encode(["GenericUpdate Failed Articleid:- ".$articleId => $e->getMessage()]);
            $this->logTracking->print_statement($message);
            return ;
        }


    }
}
